/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/


module  traf_light2(
    clk         ,
    rst_n       ,
    led_east    ,
    led_south   ,
    led_west    ,
    led_north   
    );



parameter   COUNT_TIME      =   26'd5000_0000;
parameter   CYCLE_TIME      =   5'd30        ;
parameter   COUNT_WID       =   26           ;
parameter   SEC_WID         =   5            ;
parameter   LED_NUM         =   3            ;
parameter   STA_W           =   2            ;

parameter   STA_G           =   2'd1         ;
parameter   STA_Y           =   2'd2         ;
parameter   STA_R           =   2'd3         ;
parameter   GREEN           =   3'b110       ;
parameter   YELLOW          =   3'b101       ;
parameter   RED             =   3'b011       ;


input                   clk                 ;
input                   rst_n               ;
output  [LED_NUM-1:0]   led_east            ;
output  [LED_NUM-1:0]   led_south           ;
output  [LED_NUM-1:0]   led_west            ;
output  [LED_NUM-1:0]   led_north           ;

reg     [LED_NUM-1:0]   led_east            ;
reg     [LED_NUM-1:0]   led_south           ;
reg     [LED_NUM-1:0]   led_west            ;
reg     [LED_NUM-1:0]   led_north           ;

reg     [COUNT_WID-1:0] counter             ;
wire                    add_counter         ;
wire                    end_counter         ;
reg     [SEC_WID-1:0]   second              ;
wire                    add_second          ;
wire                    end_second          ;

reg     [STA_W-1:0]     ew_state_c            ;
reg     [STA_W-1:0]     ew_state_n            ;
wire                    idle2sta_g_start_ew   ;
wire                    sta_g2sta_y_start_ew  ;
wire                    sta_y2sta_r_start_ew  ;
wire                    sta_r2sta_g_start_ew  ;

reg     [STA_W-1:0]     sn_state_c            ;
reg     [STA_W-1:0]     sn_state_n            ;
wire                    idle2sta_r_start_sn   ;
wire                    sta_r2sta_g_start_sn  ;
wire                    sta_g2sta_y_start_sn  ;
wire                    sta_y2sta_r_start_sn  ;


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        counter <= 0; 
    end
    else if(add_counter) begin
        if(end_counter)
            counter <= 0; 
        else
            counter <= counter+1 ;
   end
end
assign add_counter = 1;
assign end_counter = add_counter  && counter == COUNT_TIME-1 ;


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        second <= 0; 
    end
    else if(add_second) begin
        if(end_second)
            second <= 0; 
        else
            second <= second+1 ;
   end
end
assign add_second = end_counter;
assign end_second = add_second  && second == CYCLE_TIME-1 ;


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        ew_state_c <= STA_G ;
    end
    else begin
        ew_state_c <= ew_state_n;
   end
end

always @(*) begin 
    case(ew_state_c)  
        STA_G :begin
            if(sta_g2sta_y_start_ew) 
                ew_state_n = STA_Y ;
            else 
                ew_state_n = ew_state_c ;
        end
        STA_Y :begin
            if(sta_y2sta_r_start_ew) 
                ew_state_n = STA_R ;
            else 
                ew_state_n = ew_state_c ;
        end
        STA_R :begin
            if(sta_r2sta_g_start_ew) 
                ew_state_n = STA_G ;
            else 
                ew_state_n = ew_state_c ;
        end
        default : ew_state_n = STA_G ;
    endcase
end

assign sta_g2sta_y_start_ew = ew_state_c==STA_G && add_second  && second == 10-1;
assign sta_y2sta_r_start_ew = ew_state_c==STA_Y && add_second  && second == 15-1;
assign sta_r2sta_g_start_ew = ew_state_c==STA_R && add_second  && second == 30-1;

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        led_east<=GREEN; 
    end
    else if(ew_state_c==STA_G)begin
        led_east<=GREEN;
    end
    else if(ew_state_c==STA_Y)begin
        led_east<=YELLOW;
    end
    else if(ew_state_c==STA_R)begin
        led_east<=RED;
    end
    else begin
        led_east<=GREEN;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        led_west<=GREEN; 
    end
    else if(ew_state_c==STA_G)begin
        led_west<=GREEN;
    end
    else if(ew_state_c==STA_Y)begin
        led_west<=YELLOW;
    end
    else if(ew_state_c==STA_R)begin
        led_west<=RED;
    end
    else begin
        led_west<=GREEN;
    end
end



always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        sn_state_c <= STA_R ;
    end
    else begin
        sn_state_c <= sn_state_n;
   end
end

always @(*) begin 
    case(sn_state_c)  
        STA_R :begin
            if(sta_r2sta_g_start_sn) 
                sn_state_n = STA_G ;
            else 
                sn_state_n = sn_state_c ;
        end
        STA_G :begin
            if(sta_g2sta_y_start_sn) 
                sn_state_n = STA_Y ;
            else 
                sn_state_n = sn_state_c ;
        end
        STA_Y :begin
            if(sta_y2sta_r_start_sn) 
                sn_state_n = STA_R ;
            else 
                sn_state_n = sn_state_c ;
        end
        default : sn_state_n = STA_R ;
    endcase
end

assign sta_r2sta_g_start_sn = sn_state_c==STA_R && add_second  && second == 15-1;
assign sta_g2sta_y_start_sn = sn_state_c==STA_G && add_second  && second == 25-1;
assign sta_y2sta_r_start_sn = sn_state_c==STA_Y && add_second  && second == 30-1;

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        led_south<=RED; 
    end
    else if(sn_state_c==STA_G)begin
        led_south<=GREEN;
    end
    else if(sn_state_c==STA_Y)begin
        led_south<=YELLOW;
    end
    else if(sn_state_c==STA_R)begin
        led_south<=RED;
    end
    else begin
        led_south<=GREEN;     
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        led_north<=RED; 
    end
    else if(sn_state_c==STA_G)begin
        led_north<=GREEN;
    end
    else if(sn_state_c==STA_Y)begin
        led_north<=YELLOW;
    end
    else if(sn_state_c==STA_R)begin
        led_north<=RED;
    end
    else begin
        led_north<=GREEN;   
    end
end


endmodule

